//+------------------------------------------------------------------+
//|                                                         EA_1.mq4 |
//|                                            B. Sendra Tanuwidjaja |
//|                                               sendra_t@yahoo.com |
//+------------------------------------------------------------------+
#property copyright "B. Sendra Tanuwidjaja"
#property link      "sendra_t@yahoo.com"

//---- input parameters
extern double Lots;
extern int TrailingStop;
extern int TP;

bool CrossedUp;
bool CrossedDown;
bool BouncedUp;
bool BouncedDown;

int CrossEffectiveTime = 4;
int CrossedUpTimer;
int CrossedDownTimer;
int BouncedUpTimer;
int BouncedDownTimer;
int magicnum;
int range;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
   if(Symbol()=="GBPUSDm")
     {
      TrailingStop=45;
      magicnum=3377;
      TP = 116;
      range = 10;
     }
   if(Symbol()=="EURUSDm")
     {
      TrailingStop=35;
      magicnum=7733;
      TP = 75;
      range = 11;
     }
   if(Symbol()=="USDJPYm")
     {
      TrailingStop=35;
      magicnum=3737;
      TP = 305;
      range = 5;
     }
   if(Symbol()=="USDCHFm")
     {
      TrailingStop=25;
      magicnum=7373;
      TP = 34;
      range = 10;
     }
     
   // minimum equity USD 1,000
   Lots=NormalizeDouble(AccountEquity()*0.005,1);
     
   // No cross
   CrossedUp = false;
   CrossedDown = false;
   BouncedUp = false;
   BouncedDown = false;
   
   // Reset timer
   CrossedUpTimer = 0;
   CrossedDownTimer = 0;
   BouncedUpTimer = 0;
   BouncedDownTimer = 0;
//----

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   CrossedUp=false;
   CrossedDown=false;
   BouncedUp=false;
   BouncedDown=false;
   
   CrossedUpTimer = 0;
   CrossedDownTimer = 0;
   BouncedUpTimer = 0;
   BouncedDownTimer = 0;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   double x1=iMA(NULL,PERIOD_H1,24,0,MODE_EMA,PRICE_MEDIAN,0);
   double y1=iMA(NULL,PERIOD_H1,28,0,MODE_EMA,PRICE_MEDIAN,0);
   double z=(x1+y1)/2;
   double lineup=z+range*Point;
   double linedown=z-range*Point;
   double signalnow=iMA(NULL,PERIOD_H1,1,0,MODE_EMA,PRICE_MEDIAN,0);
   double X=iMA(NULL,PERIOD_H1,24,0,MODE_EMA,PRICE_MEDIAN,1);
   double Y=iMA(NULL,PERIOD_H1,28,0,MODE_EMA,PRICE_MEDIAN,1);
   double currentma=(X+Y)/2;
   double currentsignal=iMA(NULL,PERIOD_H1,1,0,MODE_EMA,PRICE_MEDIAN,1);
   double triggerbuy=currentma+range*Point;
   double triggersell=currentma-range*Point;
   double x=iMA(NULL,PERIOD_H1,24,0,MODE_EMA,PRICE_MEDIAN,2);
   double y=iMA(NULL,PERIOD_H1,28,0,MODE_EMA,PRICE_MEDIAN,2);
   double lastma=(x+y)/2;
   double lastup=lastma+range*Point;
   double lastdown=lastma-range*Point;
   double lastsignal=iMA(NULL,PERIOD_H1,1,0,MODE_EMA,PRICE_MEDIAN,2);
   
   if(lastsignal<=lastdown && currentsignal>=currentma && signalnow>=lineup)
     {
      CrossedUp=true;
      CrossedDown=false;
     }
     
   if(CrossedUp==true)
      CrossedUpTimer++; // Record the number of bars the cross has happened
   else
      CrossedUpTimer=0; // Reset the timer once the cross is reversed
      
   if(CrossedUpTimer>=CrossEffectiveTime)
      CrossedUp=false; // Reset register when crossed 3 bars ago
      
   if(lastsignal>=lastup && currentsignal<=currentma && signalnow<=linedown)
     {
      CrossedUp=false;
      CrossedDown=true;
     }
     
   if(CrossedDown==true)
      CrossedDownTimer++;
   else
      CrossedDownTimer=0;
      
   if(CrossedDownTimer>=CrossEffectiveTime)
      CrossedDown=false;
      
   if(lastsignal>=lastup && currentsignal<=currentma && signalnow>=lineup)
     {
      BouncedUp=true;
      BouncedDown=false;
     }
     
   if(BouncedUp==true)
      BouncedUpTimer++;
   else
      BouncedUpTimer=0;
      
   if(BouncedUpTimer>=CrossEffectiveTime)
      BouncedUp=false;
      
   if(lastsignal<=lastdown && currentsignal>=currentma && signalnow<=linedown)
     {
      BouncedUp=false;
      BouncedDown=true;
     }
     
   if(BouncedDown==true)
      BouncedDownTimer++;
   else
      BouncedDownTimer=0;
      
   if(BouncedDownTimer>=CrossEffectiveTime)
      BouncedDown=false;
   
   int cnt, ticket, total;

   total=OrdersTotal();
   if(total==0)
     {
      if(CrossedUp==true || BouncedUp==true)
        {
         ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,3,0,Ask+Point*TP,"My order",magicnum,0,Green);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
              {
               Print("Long order opened:",OrderOpenPrice());
              }
            else
              {
               Print("Error opening Long order:",GetLastError());
               Sleep(1000);
               return(0);
              }
           }
        }
      if(CrossedDown==true || BouncedDown==true)
        {
         ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,3,0,Bid-Point*TP,"My order",magicnum,0,Blue);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
              {
               Print("Short order opened:",OrderOpenPrice());
              }
            else
              {
               Print("Error opening Short order:",GetLastError());
               Sleep(1000);
               return(0);
              }
           }
        }
     }
   RefreshRates();
   
   for(cnt=0; cnt<total; cnt++)
   if(total>0)
     {
      OrderSelect(cnt,SELECT_BY_POS,MODE_TRADES);
      if(OrderType()<=OP_SELL && OrderSymbol()==Symbol())
        {
         if(OrderType()==OP_BUY) //long position is opened
           {
            if(CrossedDown==true || BouncedDown==true)
              {
               OrderClose(OrderTicket(),OrderLots(),Bid,3,Red);
               if(ticket==0)
                 {
                  OrderSend(Symbol(),OP_SELL,Lots,Bid,3,0,Bid-Point*TP,"My reverse order",magicnum,0,Blue);
                  if(ticket>0)
                    {
                     if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
                       {
                        Print("Reverse Short order opened:",OrderOpenPrice());
                       }
                     else
                       {
                        Print("Error opening Reverse Short order:",GetLastError());
                        Sleep(1000);
                        return(0);
                       }
                    }
                 }
              }
           }
         else //short position is opened
           {
            if(CrossedUp==true || BouncedUp==true)
              {
               OrderClose(OrderTicket(),OrderLots(),Ask,3,Red);
               if(ticket==0)
                 {
                  OrderSend(Symbol(),OP_BUY,Lots,Ask,3,0,Ask+Point*TP,"My reverse order",magicnum,0,Green);
                  if(ticket>0)
                    {
                     if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
                       {
                        Print("Reverse Long order opened:",OrderOpenPrice());
                       }
                     else
                       {
                        Print("Error opening Reverse Long order:",GetLastError());
                        Sleep(1000);
                        return(0);
                       }
                    }
                 }
              }
           }
        }
     }
   RefreshRates();
   //+-------------------------------------------------------------------+
   //| Checking trailing stop                                            |
   //+-------------------------------------------------------------------+
   void CheckTrailingStop;

   for(cnt=0; cnt<OrdersTotal(); cnt++)
      {
       if(OrderSelect(cnt,SELECT_BY_POS,MODE_TRADES)==false)
          break;
       if(OrderSymbol()!=Symbol())
          continue;
       if(OrderType()==OP_BUY)
         {
          if(Bid-OrderOpenPrice()>Point*TrailingStop && OrderStopLoss()<Bid-Point*TrailingStop)
             OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,GreenYellow);
         }
       if(OrderType()==OP_SELL)
         {
          if(OrderOpenPrice()-Ask>Point*TrailingStop && OrderStopLoss()>Ask+Point*TrailingStop)
             OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,GreenYellow);
         }
      }
      
   int setTrailingStop;

   if(TrailingStop>0)
     {
      OrderSelect(magicnum,SELECT_BY_TICKET);
      if(Bid-OrderOpenPrice()>Point*TrailingStop)
        {
         if(OrderStopLoss()<Bid-Point*TrailingStop)
           {
            OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Yellow);
            return(0);
           }
        }
      }
         
//----
   return(0);
  }
//+------------------------------------------------------------------+